﻿using CommandLine;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace OsdExample_CSharp
{
    class Program
    {
        enum Action_Type
        {
            GET_BLI = 0,
            SET_BLI = 1,
            GET_LSC = 2,
            SET_LSC = 3,
            GET_TCB = 4,
            SET_TCB = 5,
            GET_FUC = 6,
            SET_FUC = 7,
            GET_LED = 8,
            SET_LED = 9,
            GET_PTC = 10,
            SET_PTC = 11
        };

        public struct Action_Struct
        {
            public int type;
            public int setValue;
            public int mode;
            public StringBuilder returnValue;
        };
        class Options
        {
            [Option('l', "light-sensor", Default = false, HelpText = "-l Get light sensor mode \n-l <#light sensor mode>\n" +
                 "		0 --> disable\n" +
                 "		1 --> auto mode 1\n" +
                 "		1 --> auto mode 2"
                 )]
            public bool LightSensor { get; set; }

            [Option('b', "brightness", Default = false, HelpText = "-b Get brightness value \n-b <#brightness value> (0-10)")]
            public bool Brightness { get; set; }

            [Option('t', "keypad", Default = false, HelpText = " -t Get keypade assignment \n-t <#mode>\n" +
              " 	0 --> off\n" +
              "		1 --> On"
              )]
            public bool TouchControlButton { get; set; }

            [Option('f', "function-button", Default = false, HelpText = " -f Get function button assignment \n-f <#function button>\n" +
              " 	0 --> power button\n" +
              "		1 --> touch button\n" +
              "		2 --> function button"
              )]
            public bool FunctionButton { get; set; }

            [Option('u', "led-assignment", Default = false, HelpText = " -u Get led assignment\n -u <#LED index> <#mode>\n" +
              " 	<#LED index>\n" +
              " 	0 --> led 1\n" +
              "		1 --> led 2\n" +
              "		<#mode>\n" +
              " 	0 --> Disable\n" +
              "		1 --> LAN 1\n" +
              "		2 --> LAN 2\n" +
              "		3 --> LAN 3\n" +
              "		4 --> LAN 4\n" +
              "		5 --> UART 1 TX\n" +
              "		6 --> UART 2 TX\n" +
              "		7 --> UART 3 TX\n" +
              "		8 --> UART 1 RX\n" +
              "		9 --> UART 2 RX\n" +
              "		10 --> UART 3 RX"

              )]
            public bool LedMode { get; set; }

            [Option('p', "panel-touch-assignment", Default = false, HelpText = "-l Get panel touch assignment \n-l <#mode>\n" +
                 "		0 --> disable\n" +
                 "		1 --> enable"
                 )]
            public bool PanelTouchControl { get; set; }
        }

        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxGetBrightnessValue")]
        public static extern Int32 MxGetBrightnessValue(StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxSetBrightnessValue")]
        public static extern Int32 MxSetBrightnessValue(Int32 value, StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxGetLightSensorMode")]
        public static extern Int32 MxGetLightSensorMode(StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxSetLightSensorMode")]
        public static extern Int32 MxSetLightSensorMode(Int32 value, StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxGetTouchControl")]
        public static extern Int32 MxGetTouchControl(StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxSetTouchControl")]
        public static extern Int32 MxSetTouchControl(Int32 value, StringBuilder res, Int32 mode);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxGetFunctionButton")]
        public static extern Int32 MxGetFunctionButton(StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxSetFunctionButton")]
        public static extern Int32 MxSetFunctionButton(Int32 value, StringBuilder res);

        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxGetPanelTouchControl")]
        public static extern Int32 MxGetPanelTouchControl(StringBuilder res);
        [DllImport("MxOsd.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "MxSetPanelTouchControl")]
        public static extern Int32 MxSetPanelTouchControl(Int32 value, StringBuilder res);

        [DllImport("MxComLED.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "GetLEDMode")]
        public static extern Int32 GetLEDMode(Int32 ledIndex, ref Int32 ledMode);

        [DllImport("MxComLED.dll", SetLastError = true, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "SetLEDMode")]
        public static extern Int32 SetLEDMode(Int32 ledIndex, Int32 ledMode);

        public static string inputValue = string.Empty;

        public static int inputnumber = 0;

        public static int inputmode = -1;

        static void Main(string[] args)
        {
            if (args.Length != 0)
            {

                if (args.Length > 1)
                {
                    inputValue = args[1];
                    Int32.TryParse(inputValue, out inputnumber);
                }

                if (args.Length > 2)
                {

                    inputValue = args[2];
                    Int32.TryParse(inputValue, out inputmode);
                }
                CommandLine.Parser.Default.ParseArguments<Options>(args)
    .WithParsed(RunOptions)
    .WithNotParsed(HandleParseError);
            }
        }
        static void RunOptions(Options opts)
        {
            Action_Struct action = new Action_Struct();
            action.setValue = inputnumber;
            action.mode = inputmode;
            if (opts.Brightness)
            {
                if (inputValue != string.Empty)
                    action.type = (int)Action_Type.SET_BLI;
                else
                    action.type = (int)Action_Type.GET_BLI;
            }
            else if (opts.LightSensor)
            {
                if (inputValue != string.Empty)
                    action.type = (int)Action_Type.SET_LSC;
                else
                    action.type = (int)Action_Type.GET_LSC;
            }
            else if (opts.TouchControlButton)
            {

                if (inputValue != string.Empty)
                    if (inputmode == -1)
                    {
                        Console.WriteLine("Input parameter error");
                        Environment.Exit(1);
                    }
                    else
                    {
                        action.type = (int)Action_Type.SET_TCB;
                    }
                else
                    action.type = (int)Action_Type.GET_TCB;
            }
            else if (opts.FunctionButton)
            {
                if (inputValue != string.Empty)
                    action.type = (int)Action_Type.SET_FUC;
                else
                    action.type = (int)Action_Type.GET_FUC;
            }
            else if (opts.LedMode)
            {
                if (inputValue != string.Empty)
                    if (inputmode == -1)
                    {
                        Console.WriteLine("Input parameter error");
                        Environment.Exit(1);
                    }
                    else
                    {
                        action.type = (int)Action_Type.SET_LED;
                    }
                else
                    action.type = (int)Action_Type.GET_LED;
            }
            else if (opts.PanelTouchControl)
            {
                if (inputValue != string.Empty)
                    action.type = (int)Action_Type.SET_PTC;
                else
                    action.type = (int)Action_Type.GET_PTC;
            }

            DoAction(action);
        }
        static void HandleParseError(IEnumerable<Error> errs)
        {
            //handle errors
        }

        static void DoAction(Action_Struct action)
        {
            int ret;
            int ledMode = 0;
            action.returnValue = new StringBuilder(512);
            switch (action.type)
            {
                default:
                    ret = -1;
                    break;
                case (int)Action_Type.GET_BLI:
                    ret = MxGetBrightnessValue(action.returnValue);
                    Console.WriteLine("Brightness value: {0}", action.returnValue);
                    break;
                case (int)Action_Type.SET_BLI:
                    ret = MxSetBrightnessValue(action.setValue, action.returnValue);
                    Console.WriteLine("Brightness value: {0}", action.returnValue);
                    break;
                case (int)Action_Type.GET_LSC:
                    MxGetLightSensorMode(action.returnValue);
                    Console.WriteLine("Light sensor mode assignment: {0}", action.returnValue);
                    break;
                case (int)Action_Type.SET_LSC:
                    MxSetLightSensorMode(action.setValue, action.returnValue);
                    Console.WriteLine("Light sensor mode assignment: {0}", action.returnValue);
                    break;
                case (int)Action_Type.GET_TCB:
                    int pwr;
                    int brPlus;
                    int brMinus;

                    if ((ret = MxGetTouchControl(action.returnValue)) != 0)
                    {
                        Console.WriteLine("Get keypad assignment fail.");
                    }
                    else
                    {
                        brMinus = Convert.ToByte(action.returnValue.ToString()) >> 2;
                        brPlus = Convert.ToByte(action.returnValue.ToString()) >> 3;
                        pwr = Convert.ToByte(action.returnValue.ToString()) >> 4;

                        Console.WriteLine("Keypad assignment: {0}", (brMinus & brPlus & pwr));
                    }
                    break;
                case (int)Action_Type.SET_TCB:
                    // 0: power, 1: br+, 2: br-, 3: all
                    MxSetTouchControl(3, action.returnValue, action.setValue);
                    Console.WriteLine("Keypad assignment: {0}", action.returnValue);
                    break;
                case (int)Action_Type.GET_FUC:
                    MxGetFunctionButton(action.returnValue);
                    Console.WriteLine("Function button assignment: {0}", action.returnValue);
                    break;
                case (int)Action_Type.SET_FUC:
                    MxSetFunctionButton(action.setValue, action.returnValue);
                    Console.WriteLine("Function button assignment: {0}", action.returnValue);
                    break;
                case (int)Action_Type.GET_LED:
                    GetLEDMode(0, ref ledMode);
                    Console.WriteLine("LED 0 assignment: {0}", ledMode);
                    GetLEDMode(1, ref ledMode);
                    Console.WriteLine("LED 1 assignment: {0}", ledMode);
                    break;
                case (int)Action_Type.SET_LED:
                    SetLEDMode(action.setValue, (int)action.mode);
                    break;
                case (int)Action_Type.GET_PTC:
                    MxGetPanelTouchControl(action.returnValue);
                    Console.WriteLine("Panel touch assignment: {0}", action.returnValue);
                    break;
                case (int)Action_Type.SET_PTC:
                    MxSetPanelTouchControl(action.setValue, action.returnValue);
                    Console.WriteLine("Panel touch assignment: {0}", action.returnValue);
                    break;
            }
        }
    }
}
