/*
	Copyright (C) MOXA Inc. All rights reserved.

	This software is distributed under the terms of the
	MOXA License.  See the file COPYING-MOXA for details.
*/

#include <tchar.h>
#include "MxComLED.h"
#include "MxOsd.h"
#include <getopt.h>  
#include <stdio.h>
#include <bitset>
#pragma comment(lib,"MxComLED.lib") 
#pragma comment(lib,"MxOsd.lib") 
#pragma comment(lib,"getopt.lib") 
using namespace std;

#define UNSET 	-1

enum action_type {
	GET_BLI = 0,
	SET_BLI = 1,
	GET_LSC = 2,
	SET_LSC = 3,
	GET_TCB = 4,
	SET_TCB = 6,
	GET_FUC = 7,
	SET_FUC = 8,
	GET_LED = 9,
	SET_LED = 10,
	GET_PTC = 11,
	SET_PTC = 12,
};

struct action_struct {
	int type;
	int setValue;
	int mode;
	char returnValue[512];
};

void usage(FILE* fp)
{
	fprintf(fp, "Usage:\n");
	fprintf(fp, "	mx-expc-panel-ctl -b <0-10>\n");
	fprintf(fp, "OPTIONS:\n");
	fprintf(fp, "	-l \n");
	fprintf(fp, "		Get light sensor mode assignment\n");
	fprintf(fp, "	-l <#light_sensor_mode>\n");
	fprintf(fp, "		Set lightSensor mode (0: disable, 1: auto mode 1, 2: auto mode 2) \n");
	fprintf(fp, "	-b \n");
	fprintf(fp, "		Get brightness value \n");
	fprintf(fp, "	-b <#brightness_value>\n");
	fprintf(fp, "		Set brightness value: [0-10] \n");
	fprintf(fp, "	-t \n");
	fprintf(fp, "		Get keypad assignment \n");
	fprintf(fp, "	-t <#keypad_mode>\"\n");
	fprintf(fp, "		Set keypade assignment (0: off, 1: on) \n");
	fprintf(fp, "	-f \n");
	fprintf(fp, "		Get function button assignment\n");
	fprintf(fp, "	-f \"<#function_button>\"\n");
	fprintf(fp, "		Set function button (0: power button, 1: touch screen button, 2: customized button)\n");
	fprintf(fp, "	-u <#led_index>\n");
	fprintf(fp, "		Get led assignment \n");
	fprintf(fp, "	-u \"<#led_index> <#mode>\"\n");
	fprintf(fp, "		Set led assignment (0: disable, 1: LAN_1, 2: LAN_2, 3: LAN_3, 4: LAN_4\n");
	fprintf(fp, "		                5: UART_1_TX, 6: UART_2_TX, 7: UART_3_TX\n");
	fprintf(fp, "		                8: UART_1_RX, 9: UART_2_RX, 10: UART_3_RX\n");
	fprintf(fp, "	-p \n");
	fprintf(fp, "		Get panel touch assignment\n");
	fprintf(fp, "	-p <#panel touch mode>\n");
	fprintf(fp, "		Set panel touch assignment (0: off, 1: on)\n");
	fprintf(fp, "\n");
	fprintf(fp, "Example:\n");
	fprintf(fp, "	Get brightness value\n");
	fprintf(fp, "	# mx-expc-panel-ctl -b\n");
	fprintf(fp, "	Set brightness value 10\n");
	fprintf(fp, "	# mx-expc-panel-ctl -b 10\n");
	fprintf(fp, "	Get keypad assignment\n");
	fprintf(fp, "	# mx-expc-panel-ctl -t\"\n");
	fprintf(fp, "	Set keypad assignment <on>\n");
	fprintf(fp, "	# mx-expc-panel-ctl -t 1\n");
	fprintf(fp, "\n");
}

int my_atoi(const char* nptr, int* number)
{
	int tmp;

	tmp = atoi(nptr);
	if (tmp != 0) {
		*number = tmp;
		return 0;
	}
	else {
		if (!strcmp(nptr, "0")) {
			*number = 0;
			return 0;
		}
	}
	return -1;
}
void do_action(struct action_struct action)
{
	int n = 0, value = 0, mode = 0;
	int returnStatus = 0;
	char responseValue[512];

	switch (action.type) {
	case GET_BLI:

		MxGetBrightnessValue(action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to get brightness value\n");
			exit(1);
		}
		printf("Brightness value: %s\n", action.returnValue);
		break;
	case SET_BLI:
		MxSetBrightnessValue(action.setValue, action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to set brightness value\n");
			exit(1);
		}
		printf("Brightness value: %s\n", action.returnValue);
		break;
	case GET_LSC:

		MxGetLightSensorMode(action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to get light sensor assignment\n");
			exit(1);
		}
		printf("Light sensor assignment: %s\n", action.returnValue);
		break;
	case SET_LSC:
		MxSetLightSensorMode(action.setValue, action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to set light sensor mode\n");
			exit(1);
		}
		printf("Light sensor assignment: %s\n", action.returnValue);
		break;
	case GET_TCB:

		if (MxGetTouchControl(action.returnValue) != 0)
		{
			fprintf(stderr, "Fail to get keypad assignment\n");
		}
		else
		{
			int brMinus = std::bitset<8>(atoi(action.returnValue))[2];
			int brPlus = std::bitset<8>(atoi(action.returnValue))[3];
			int pwr = std::bitset<8>(atoi(action.returnValue))[4];
			
			printf("Keypad assignment: %d\n", (brMinus & brPlus & pwr));
		}
		break;
	case SET_TCB:
		// 0: power, 1: br+, 2: br-, 3: all
		MxSetTouchControl(3, action.returnValue, action.setValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to set keypad assignment\n");
			exit(1);
		}
		if (MxGetTouchControl(action.returnValue) != 0)
		{
			fprintf(stderr, "Fail to get keypad assignment\n");
		}
		else
		{
			int brMinus = std::bitset<8>(atoi(action.returnValue))[2];
			int brPlus = std::bitset<8>(atoi(action.returnValue))[3];
			int pwr = std::bitset<8>(atoi(action.returnValue))[4];

			printf("Keypad assignment: %d\n", (brMinus & brPlus & pwr));
		}
		break;
	case GET_FUC:

		MxGetFunctionButton(action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to get function button assignment\n");
			exit(1);
		}
		printf("Function button assignment: %s\n", action.returnValue);
		break;
	case SET_FUC:
		MxSetFunctionButton(action.setValue, action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to set function button assignment\n");
			exit(1);
		}
		printf("function button assignment: %s\n", action.returnValue);
		break;
	case GET_LED:
		GetLEDMode(0, mode);
		if (mode < 0) {
			fprintf(stderr, "Fail to get led 1 assignment\n");
			exit(1);
		}
		printf("LED 1 assignment: %d\n", mode);
		GetLEDMode(1, mode);
		if (mode < 0) {
			fprintf(stderr, "Fail to get led 2 assignment\n");
			exit(1);
		}
		printf("LED 2 assignment: %d\n", mode);
		break;
	case SET_LED:
		if (SetLEDMode(action.setValue, action.mode)  < 0) {
			fprintf(stderr, "Fail to assign LED mode\n");
			exit(1);
		}
		break;
	case GET_PTC:
		MxGetPanelTouchControl(action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to get panel touch assignment\n");
			exit(1);
		}
		printf("Panel touch assignment: %s\n", action.returnValue);
		break;
	case SET_PTC:
		MxSetPanelTouchControl(action.setValue, action.returnValue);
		if (action.returnValue < 0) {
			fprintf(stderr, "Fail to set panel touch assignment\n");
			exit(1);
		}
		printf("Panel touch assignment: %s\n", action.returnValue);
		break;
	}
}
int main(int argc, char** argv)
{
	struct action_struct action = { UNSET, UNSET, UNSET };
	int c;

	while (1) {
		if (argv)
			c = getopt(argc, argv, "hb::l::t::f::u::p::");

		if (c == -1)
			break;
		switch (c) {
		case 'h':
			usage(stdout);
			exit(0);
		case 'b':
			if (optarg == NULL)
			{
				action.type = GET_BLI;
			}
			else
			{
				if (my_atoi(optarg, &action.setValue) != 0) {
					fprintf(stderr, "%s is not a number\n", optarg);
					exit(1);
				}
				action.type = SET_BLI;
			}
			break;
		case 'l':
			if (optarg == NULL)
			{
				action.type = GET_LSC;
			}
			else
			{
				if (my_atoi(optarg, &action.setValue) != 0) {
					fprintf(stderr, "%s is not a number\n", optarg);
					exit(1);
				}
				action.type = SET_LSC;
			}
			break;
		case 't':
			if (optarg == NULL)
			{
				action.type = GET_TCB;

			}
			else
			{
				if (my_atoi(optarg, &action.setValue) != 0) {
					fprintf(stderr, "%s is not a number\n", optarg);
					exit(1);
				}
				action.type = SET_TCB;
			}
			break;
		case 'f':
			if (optarg == NULL)
			{
				action.type = GET_FUC;
			}
			else
			{
				if (my_atoi(optarg, &action.setValue) != 0) {
					fprintf(stderr, "%s is not a number\n", optarg);
					exit(1);
				}
				action.type = SET_FUC;
			}
			break;
		case 'u':
			if (optarg == NULL)
			{
				action.type = GET_LED;

			}
			else
			{
				char* pch1 = strtok(optarg, " ");
				char* pch2 = strtok(NULL, "\0");
				if (pch2 == NULL)
				{
					fprintf(stderr, "%s mode is not a number\n", pch2);
					exit(1);

				}
				if (my_atoi(pch1, &action.setValue) != 0) {
					fprintf(stderr, "%s value is not a number\n", pch1);
					exit(1);
				}

				if (my_atoi(pch2, &action.mode) != 0) {
					fprintf(stderr, "%s mode is not a number\n", pch2);
					exit(1);
				}
				action.type = SET_LED;

			}
			break;
		case 'p':
			if (optarg == NULL)
			{
				action.type = GET_PTC;
			}
			else
			{
				if (my_atoi(optarg, &action.setValue) != 0) {
					fprintf(stderr, "%s is not a number\n", optarg);
					exit(1);
				}
				action.type = SET_PTC;
			}
			break;
		default:
			usage(stderr);
			exit(99);
		}
	}

	if (optind < argc) {
		fprintf(stderr, "non-option arguments found: ");
		while (optind < argc)
			fprintf(stderr, "%s ", argv[optind++]);
		fprintf(stderr, "\n");

		usage(stderr);
		exit(99);
	}

	do_action(action);

	exit(0);
}

